@@ -45,37 +45,30 @@ const PLAYBACK_TYPES: [&rspotify::model::AdditionalType; 2] = [
 #[derive(Clone)]
 pub struct AppClient {
     http: reqwest::Client,
-    /// The integrated Spotify client used by the application
+    /// The integrated Spotify client, mainly used for streaming and librespot integration
     spotify: Arc<spotify::Spotify>,
     auth_config: AuthConfig,
-    /// The user-provided Spotify client, mainly for Spotify Connect integration
+    /// The user-provided Spotify client, mainly used for interacting with Spotify Web APIs
     user_client: Option<rspotify::AuthCodePkceSpotify>,
     #[cfg(feature = "streaming")]
     stream_conn: Arc<Mutex<Option<librespot_connect::Spirc>>>,
 }
 
 impl Deref for AppClient {
-    // this should use `spotify` to get full API access because user-provided client has limited scopes
-    // TODO: revert after https://github.com/aome510/spotify-player/issues/890 is resolved
     type Target = rspotify::AuthCodePkceSpotify;
     fn deref(&self) -> &Self::Target {
-        self.user_client.as_ref().unwrap()
+        self.user_client
+            .as_ref()
+            .expect("user-provided client should be initialized")
     }
 }
 
-fn market_query() -> Query<'static> {
-    Query::from([("market", "from_token")])
-}
-
 impl AppClient {
     /// Construct a new client
     pub async fn new() -> Result<Self> {
         let configs = config::get_config();
         let auth_config = AuthConfig::new(configs)?;
 
-        // Construct user-provided client.
-        // This custom client is needed for Spotify Connect integration because the custom Spotify client (`AppClient::spotify`),
-        // which `spotify-player` uses to retrieve Spotify data from official API server, doesn't have access to user available devices
         let mut user_client = configs.app_config.get_user_client_id()?.clone().map(|id| {
             let creds = rspotify::Credentials { id, secret: None };
             let mut scopes = auth::OAUTH_SCOPES
@@ -706,14 +699,7 @@ impl AppClient {
 
     /// Get user available devices
     pub async fn available_devices(&self) -> Result<Vec<rspotify::model::Device>> {
-        match &self.user_client {
-            None => {
-                tracing::warn!("User-provided client integration is not enabled, no device found.");
-                tracing::warn!("Please make sure you setup Spotify Connect as described in https://github.com/aome510/spotify-player#spotify-connect.");
-                Ok(vec![])
-            }
-            Some(client) => Ok(client.device().await?),
-        }
+        Ok(self.device().await?)
     }
 
     pub fn update_playback(&self, state: &SharedState) {
@@ -749,11 +735,41 @@ impl AppClient {
 
     /// Get Spotify's available browse playlists of a given category
     pub async fn browse_category_playlists(&self, category_id: &str) -> Result<Vec<Playlist>> {
-        let first_page = self
-            .category_playlists_manual(category_id, None, Some(50), None)
-            .await?;
+        // TODO: this should use `rspotify::category_playlists_manual` API instead of `http_get`
+        // The current implementation is a workaround for https://github.com/ramsayleung/rspotify/issues/535
 
-        Ok(first_page.items.into_iter().map(Playlist::from).collect())
+        // Ok(self
+        //     .category_playlists_manual(
+        //         category_id,
+        //         Some(rspotify::model::Market::FromToken),
+        //         Some(50),
+        //         None,
+        //     )
+        //     .await?
+        //     .items
+        //     .into_iter()
+        //     .map(Into::into)
+        //     .collect())
+
+        #[derive(Deserialize, Debug)]
+        struct BrowseCategoryPlaylistsResponse {
+            playlists: rspotify::model::Page<serde_json::Value>,
+        }
+
+        Ok(self
+            .http_get::<BrowseCategoryPlaylistsResponse>(
+                &format!("{SPOTIFY_API_ENDPOINT}/browse/categories/{category_id}/playlists"),
+                &Query::from([("limit", "50")]),
+            )
+            .await?
+            .playlists
+            .items
+            .into_iter()
+            .filter_map(|item| {
+                serde_json::from_value::<rspotify::model::SimplifiedPlaylist>(item).ok()
+            })
+            .map(Into::into)
+            .collect())
     }
 
     /// Find an available device. If found, return the device's ID.
@@ -806,14 +822,13 @@ impl AppClient {
 
     /// Get the saved (liked) tracks of the current user
     pub async fn current_user_saved_tracks(&self) -> Result<Vec<Track>> {
-        let first_page = self
-            .current_user_saved_tracks_manual(
-                Some(rspotify::model::Market::FromToken),
-                Some(50),
-                None,
+        let tracks = self
+            .all_paging_items::<rspotify::model::SavedTrack>(
+                &format!("{SPOTIFY_API_ENDPOINT}/me/tracks"),
+                0, // we don't know the total number of saved tracks beforehand
             )
             .await?;
-        let tracks = self.all_paging_items(first_page, &market_query()).await?;
+
         Ok(tracks
             .into_iter()
             .filter_map(|t| Track::try_from_full_track(t.track))
@@ -840,11 +855,13 @@ impl AppClient {
 
     /// Get the top tracks of the current user
     pub async fn current_user_top_tracks(&self) -> Result<Vec<Track>> {
-        let first_page = self
-            .current_user_top_tracks_manual(None, Some(50), None)
+        let tracks = self
+            .all_paging_items::<rspotify::model::FullTrack>(
+                &format!("{SPOTIFY_API_ENDPOINT}/me/top/tracks"),
+                0, // we don't know the total number of top tracks beforehand
+            )
             .await?;
 
-        let tracks = self.all_paging_items(first_page, &Query::new()).await?;
         Ok(tracks
             .into_iter()
             .filter_map(Track::try_from_full_track)
@@ -853,21 +870,12 @@ impl AppClient {
 
     /// Get all playlists of the current user
     pub async fn current_user_playlists(&self) -> Result<Vec<Playlist>> {
-        // TODO: this should use `rspotify::current_user_playlists_manual` API instead of `internal_call`
-        // See: https://github.com/ramsayleung/rspotify/issues/459
-        // Fetch the first page of playlists
-        let first_page = self
-            .http_get::<rspotify::model::Page<rspotify::model::SimplifiedPlaylist>>(
+        let playlists = self
+            .all_paging_items::<rspotify::model::SimplifiedPlaylist>(
                 &format!("{SPOTIFY_API_ENDPOINT}/me/playlists"),
-                &Query::from([("limit", "50")]),
+                0, // we don't know the total number of playlists beforehand
             )
             .await?;
-        // let first_page = self
-        //     .current_user_playlists_manual(Some(50), None)
-        //     .await?;
-
-        // Fetch all pages of playlists
-        let playlists = self.all_paging_items(first_page, &Query::new()).await?;
 
         Ok(playlists
             .into_iter()
@@ -901,62 +909,44 @@ impl AppClient {
 
     /// Get all saved albums of the current user
     pub async fn current_user_saved_albums(&self) -> Result<Vec<Album>> {
-        let first_page = self
-            .current_user_saved_albums_manual(
-                Some(rspotify::model::Market::FromToken),
-                Some(50),
-                None,
+        let albums = self
+            .all_paging_items::<rspotify::model::SavedAlbum>(
+                &format!("{SPOTIFY_API_ENDPOINT}/me/albums"),
+                0, // we don't know the total number of saved albums beforehand
             )
             .await?;
 
-        let albums = self.all_paging_items(first_page, &Query::new()).await?;
-
         // Converts `rspotify::model::SavedAlbum` into `state::Album`
         Ok(albums.into_iter().map(Album::from).collect())
     }
 
     /// Get all saved shows of the current user
     pub async fn current_user_saved_shows(&self) -> Result<Vec<Show>> {
-        let first_page = self.get_saved_show_manual(Some(50), None).await?;
-        let shows = self.all_paging_items(first_page, &Query::new()).await?;
+        let shows = self
+            .all_paging_items::<rspotify::model::Show>(
+                &format!("{SPOTIFY_API_ENDPOINT}/me/shows"),
+                0, // we don't know the total number of saved shows beforehand
+            )
+            .await?;
+
         Ok(shows.into_iter().map(|s| s.show.into()).collect())
     }
 
     /// Get all albums of an artist
     pub async fn artist_albums(&self, artist_id: ArtistId<'_>) -> Result<Vec<Album>> {
-        let payload = market_query();
-
-        let mut singles = {
-            let first_page = self
-                .artist_albums_manual(
-                    artist_id.as_ref(),
-                    Some(rspotify::model::AlbumType::Single),
-                    Some(rspotify::model::Market::FromToken),
-                    Some(50),
-                    None,
-                )
-                .await?;
-            self.all_paging_items(first_page, &payload).await
-        }?;
-        let mut albums = {
-            let first_page = self
-                .artist_albums_manual(
-                    artist_id.as_ref(),
-                    Some(rspotify::model::AlbumType::Album),
-                    Some(rspotify::model::Market::FromToken),
-                    Some(50),
-                    None,
-                )
-                .await?;
-            self.all_paging_items(first_page, &payload).await
-        }?;
-        albums.append(&mut singles);
-
-        // converts `rspotify::model::SimplifiedAlbum` into `state::Album`
-        let albums = albums
+        let albums = self
+            .all_paging_items::<rspotify::model::SimplifiedAlbum>(
+                &format!(
+                    "{SPOTIFY_API_ENDPOINT}/artists/{}/albums?include_groups=album,single",
+                    artist_id.id()
+                ),
+                0, // we don't know the total number of artist albums beforehand
+            )
+            .await?
             .into_iter()
             .filter_map(Album::try_from_simplified_album)
             .collect();
+
         Ok(AppClient::process_artist_albums(albums))
     }
 
@@ -1378,22 +1368,22 @@ impl AppClient {
         let playlist_uri = playlist_id.uri();
         tracing::info!("Get playlist context: {}", playlist_uri);
 
-        // TODO: this should use `rspotify::playlist` API instead of `internal_call`
-        // See: https://github.com/ramsayleung/rspotify/issues/459
-        // let playlist = self
-        //     .playlist(playlist_id, None, Some(Market::FromToken))
-        //     .await?;
         let playlist = self
-            .http_get::<rspotify::model::FullPlaylist>(
-                &format!("{SPOTIFY_API_ENDPOINT}/playlists/{}", playlist_id.id()),
-                &market_query(),
+            .playlist(
+                playlist_id.clone(),
+                None,
+                Some(rspotify::model::Market::FromToken),
             )
             .await?;
 
-        // get the playlist's tracks
-        let first_page = playlist.tracks.clone();
         let tracks = self
-            .all_paging_items(first_page, &market_query())
+            .all_paging_items(
+                &format!(
+                    "{SPOTIFY_API_ENDPOINT}/playlists/{}/tracks",
+                    playlist_id.id(),
+                ),
+                playlist.tracks.total as usize,
+            )
             .await?
             .into_iter()
             .filter_map(Track::try_from_playlist_item)
@@ -1411,16 +1401,20 @@ impl AppClient {
         tracing::info!("Get album context: {}", album_uri);
 
         let album = self
-            .album(album_id, Some(rspotify::model::Market::FromToken))
+            .album(album_id.clone(), Some(rspotify::model::Market::FromToken))
             .await?;
-        let first_page = album.tracks.clone();
+
+        let total_tracks = album.tracks.total as usize;
 
         // converts `rspotify::model::FullAlbum` into `state::Album`
         let album: Album = album.into();
 
         // get the album's tracks
         let tracks = self
-            .all_paging_items(first_page, &Query::new())
+            .all_paging_items(
+                &format!("{SPOTIFY_API_ENDPOINT}/albums/{}/tracks", album_id.id()),
+                total_tracks,
+            )
             .await?
             .into_iter()
             .filter_map(|t| {
@@ -1453,24 +1447,20 @@ impl AppClient {
         let top_tracks = self
             .artist_top_tracks(artist_id.as_ref(), Some(rspotify::model::Market::FromToken))
             .await
-            .context("get artist's top tracks")?;
-        let top_tracks = top_tracks
+            .context("get artist's top tracks")?
             .into_iter()
             .filter_map(Track::try_from_full_track)
             .collect::<Vec<_>>();
 
-        // temporarily disable related-artists due to a rate-limiting issue
-        // TODO: revert after https://github.com/aome510/spotify-player/issues/890 is resolved
-        // #[allow(deprecated)]
-        // let related_artists = self
-        //     .artist_related_artists(artist_id.as_ref())
-        //     .await
-        //     .context("get related artists")?;
-        // let related_artists = related_artists
-        //     .into_iter()
-        //     .map(std::convert::Into::into)
-        //     .collect::<Vec<_>>();
-        let related_artists = Vec::new();
+        #[allow(deprecated)]
+        let related_artists = self
+            .artist_related_artists(artist_id.as_ref())
+            .await
+            .ok()
+            .unwrap_or_default()
+            .into_iter()
+            .map(std::convert::Into::into)
+            .collect::<Vec<_>>();
 
         let albums = self
             .artist_albums(artist_id.as_ref())
@@ -1490,33 +1480,22 @@ impl AppClient {
         let show_uri = show_id.uri();
         tracing::info!("Get show context: {}", show_uri);
 
-        let show = self.get_a_show(show_id, None).await?;
-        let first_page = show.episodes.clone();
-
-        // Copy first_page but use Page<Option<SimplifiedEpisode>> instead of Page<SimplifiedEpisode>
-        // This is a temporary fix for https://github.com/aome510/spotify-player/issues/663
-        let first_page = rspotify::model::Page {
-            items: first_page.items.into_iter().map(Some).collect(),
-            href: first_page.href,
-            limit: first_page.limit,
-            next: first_page.next,
-            offset: first_page.offset,
-            previous: first_page.previous,
-            total: first_page.total,
-        };
-
-        // converts `rspotify::model::FullShow` into `state::Show`
-        let show: Show = show.into();
+        let show = self.get_a_show(show_id.clone(), None).await?;
 
         // get the show's episodes
         let episodes = self
-            .all_paging_items(first_page, &Query::new())
+            .all_paging_items::<rspotify::model::SimplifiedEpisode>(
+                &format!("{SPOTIFY_API_ENDPOINT}/shows/{}/episodes", show_id.id()),
+                show.episodes.total as usize,
+            )
             .await?
             .into_iter()
-            .flatten()
             .map(std::convert::Into::into)
             .collect::<Vec<_>>();
 
+        // converts `rspotify::model::FullShow` into `state::Show`
+        let show: Show = show.into();
+
         Ok(Context::Show { show, episodes })
     }
 
@@ -1530,11 +1509,7 @@ impl AppClient {
         /// This function is mainly used to patch upstream API bugs , resulting in
         /// a type error when a third-party library like `rspotify` parses the response
         fn process_spotify_api_response(text: &str) -> String {
-            // See: https://github.com/ramsayleung/rspotify/issues/459
-            text.replace("\"images\":null", "\"images\":[]")
-                // See: https://github.com/aome510/spotify-player/issues/494
-                // an item's name can be null while Spotify requires it to be available
-                .replace("\"name\":null", "\"name\":\"\"")
+            text.to_string()
         }
 
         let access_token = self.token().await.context("get token")?;
@@ -1562,29 +1537,61 @@ impl AppClient {
         Ok(serde_json::from_str(&text)?)
     }
 
-    /// Get all paging items starting from a pagination object of the first page
-    async fn all_paging_items<T>(
-        &self,
-        first_page: rspotify::model::Page<T>,
-        payload: &Query<'_>,
-    ) -> Result<Vec<T>>
+    async fn all_paging_items<T>(&self, base_url: &str, mut count: usize) -> Result<Vec<T>>
     where
-        T: serde::de::DeserializeOwned,
+        T: serde::de::DeserializeOwned + std::fmt::Debug,
     {
-        let mut items = first_page.items;
-        let mut maybe_next = first_page.next;
+        const PAGE_LIMIT: usize = 50;
+        const MAX_PARALLEL: usize = 8;
 
-        while let Some(url) = maybe_next {
-            let mut next_page = self
-                .http_get::<rspotify::model::Page<T>>(&url, payload)
-                .await?;
-            if next_page.items.is_empty() {
+        let mut all_items = Vec::new();
+        let mut offset = 0;
+
+        // if count is 0 (i.e., unknown), set it to usize::MAX to fetch until no more items
+        if count == 0 {
+            count = usize::MAX;
+        }
+
+        while offset < count {
+            let n_jobs = std::cmp::min(MAX_PARALLEL, (count - offset).div_ceil(PAGE_LIMIT));
+
+            let mut futures = Vec::with_capacity(n_jobs);
+
+            for i in 0..n_jobs {
+                let current_offset = offset + i * PAGE_LIMIT;
+                let limit_str = PAGE_LIMIT.to_string();
+                let offset_str = current_offset.to_string();
+
+                futures.push(async move {
+                    let params = Query::from([
+                        ("market", "from_token"),
+                        ("limit", &limit_str),
+                        ("offset", &offset_str),
+                    ]);
+                    self.http_get::<rspotify::model::Page<T>>(base_url, &params)
+                        .await
+                });
+            }
+
+            let results = futures::future::try_join_all(futures).await?;
+
+            let mut found_empty = false;
+            for mut page in results {
+                if page.items.is_empty() {
+                    found_empty = true;
+                    break;
+                }
+                all_items.append(&mut page.items);
+            }
+
+            if found_empty {
                 break;
             }
-            items.append(&mut next_page.items);
-            maybe_next = next_page.next;
+
+            offset += n_jobs * PAGE_LIMIT;
         }
-        Ok(items)
+
+        Ok(all_items)
     }
 
     /// Get all cursor-based paging items starting from a pagination object of the first page
@@ -1662,7 +1669,6 @@ impl AppClient {
                             playback.volume = Some(volume);
                         }
                         playback.mute_state = bp.mute_state;
-                        playback.fake_track_repeat_state = bp.fake_track_repeat_state;
                     }
                     playback
                 });
@@ -1969,9 +1975,7 @@ impl AppClient {
     /// Process a list of albums, which includes
     /// - sort albums by the release date
     /// - sort albums by the type if `sort_artist_albums_by_type` config is enabled
-    fn process_artist_albums(albums: Vec<Album>) -> Vec<Album> {
-        let mut albums = albums.into_iter().collect::<Vec<_>>();
-
+    fn process_artist_albums(mut albums: Vec<Album>) -> Vec<Album> {
         albums.sort_by(|x, y| y.release_date.partial_cmp(&x.release_date).unwrap());
 
         if config::get_config().app_config.sort_artist_albums_by_type {
