@@ -55,10 +55,6 @@ impl Default for KeymapConfig {
                     key_sequence: "C-r".into(),
                     command: Command::Repeat,
                 },
-                Keymap {
-                    key_sequence: "M-r".into(),
-                    command: Command::ToggleFakeTrackRepeatMode,
-                },
                 Keymap {
                     key_sequence: "C-s".into(),
                     command: Command::Shuffle,
@@ -76,7 +72,7 @@ impl Default for KeymapConfig {
                     command: Command::Mute,
                 },
                 Keymap {
-                    key_sequence: "0".into(),
+                    key_sequence: "^".into(),
                     command: Command::SeekStart,
                 },
                 Keymap {
