@@ -22,7 +22,7 @@ use theme::ThemeConfig;
 
 pub use theme::Theme;
 
-use crate::auth::SPOTIFY_CLIENT_ID;
+use crate::auth::{NCSPOT_CLIENT_ID, SPOTIFY_CLIENT_ID};
 
 static CONFIGS: OnceLock<Configs> = OnceLock::new();
 
@@ -87,6 +87,7 @@ pub struct AppConfig {
     pub play_icon: String,
     pub pause_icon: String,
     pub liked_icon: String,
+    pub explicit_icon: String,
 
     // layout configs
     pub border_type: BorderType,
@@ -274,10 +275,14 @@ impl Default for AppConfig {
     fn default() -> Self {
         Self {
             theme: "dracula".to_owned(),
-            // NOTE: this is a workaround for people who cannot create their own Spotify application
-            // due to current Spotify restrictions.
-            // TODO: remove this once https://github.com/aome510/spotify-player/issues/890 is resolved.
-            client_id: Some("b28a574d8ad44fc7b5a89994431a8f88".to_string()),
+            // Use ncspot's client ID as a fallback for user-provided client ID
+            //
+            // Most of the time, using ncspot's client ID is better than user-provided one
+            // because it is registered with [extended quota mode] and predates [spotify API changes]
+            //
+            // [extended quota mode]: https://developer.spotify.com/documentation/web-api/concepts/quota-modes
+            // [spotify API changes]: https://developer.spotify.com/blog/2024-11-27-changes-to-the-web-api
+            client_id: Some(NCSPOT_CLIENT_ID.to_string()),
             client_id_command: None,
 
             client_port: 8080,
@@ -320,6 +325,7 @@ impl Default for AppConfig {
             pause_icon: "▌▌".to_string(),
             play_icon: "▶".to_string(),
             liked_icon: "♥".to_string(),
+            explicit_icon: "(E)".to_string(),
 
             border_type: BorderType::Plain,
             progress_bar_type: ProgressBarType::Rectangle,
