@@ -5,7 +5,10 @@ use super::{
 };
 #[cfg(feature = "image")]
 use crate::state::ImageRenderInfo;
-use crate::ui::utils::{format_genres, to_bidi_string};
+use crate::{
+    state::Track,
+    ui::utils::{format_genres, to_bidi_string},
+};
 #[cfg(feature = "image")]
 use anyhow::{Context, Result};
 use rspotify::model::Id;
@@ -154,8 +157,7 @@ pub fn render_playback_window(
     frame.render_widget(
             Paragraph::new(
                 "No playback found. Please start a new playback.\n \
-                 Make sure there is a running Spotify device and try to connect to one using the `SwitchDevice` command.\n \
-                 You may also need to set up Spotify Connect to see available devices as in https://github.com/aome510/spotify-player#spotify-connect."
+                 Make sure there is a running Spotify device and try to connect to one using the `SwitchDevice` command."
             )
             .wrap(Wrap { trim: true }),
             rect,
@@ -261,12 +263,8 @@ fn construct_playback_text(
             "{track}" => match playable {
                 rspotify::model::PlayableItem::Track(track) => (
                     {
-                        let bidi_string = to_bidi_string(&track.name);
-                        if track.explicit {
-                            format!("{bidi_string} (E)")
-                        } else {
-                            bidi_string
-                        }
+                        let track = Track::try_from_full_track(track.clone()).unwrap();
+                        to_bidi_string(&track.display_name())
                     },
                     ui.theme.playback_track(),
                 ),
@@ -335,11 +333,7 @@ fn construct_playback_text(
                 }
             },
             "{metadata}" => {
-                let repeat_value = if playback.fake_track_repeat_state {
-                    "track (fake)".to_string()
-                } else {
-                    <&'static str>::from(playback.repeat_state).to_string()
-                };
+                let repeat_value = <&'static str>::from(playback.repeat_state).to_string();
 
                 let volume_value = if let Some(volume) = playback.mute_state {
                     format!("{volume}% (muted)")
